# The following sample shows how to set problem type to "fixedLP"
# the compute dual values, and retstore back original type.
# The code uses a Python contextmanager to restore type.

from contextlib import contextmanager


@contextmanager
def model_solvefixed(mdl):
    assert mdl.solution
    cpx = mdl._get_cplex(do_raise=True, msgfn=lambda: "model_solvefixed requires CPLEX")
    # save initial problem type, to be restored.
    saved_problem_type = cpx.get_problem_type()
    if saved_problem_type == 0:
        mdl.warning("Model {0} is a LP model, solvefixed does nothing".format(mdl.name))
        return mdl
    try:
        cpx.set_problem_type(3) # 3 is constant fixed_MILP
        yield mdl
    finally:
        cpx.set_problem_type(saved_problem_type)


if __name__ == '__main__':
    from examples.delivery.modeling.nurses import build
    nm = build()
    ns = nm.solve(log_output=True)
    with model_solvefixed(nm) as nmf:
        print(f"-- problem type in block: {nm.problem_type}")
        nmf.solve(log_output=True)
        cpx = nmf.cplex
        for lc in nmf.iter_linear_constraints():
            lcd = lc.dual_value
            if lcd:
                print(f"-- constraint #{lc.index}: {str(lc)[:32]}..,  dual={lcd}")
